
# q-expansion of newform 1600.2.a.f, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1600
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-2], [0], [2], [4], [-4], [0], [4], [-2], [-2], [0], [-4], [2], [6], [-6], [4], [12], [10], [-14], [8], [8], [16], [-2], [6], [16], [-6], [14], [10], [6], [16], [-6], [12], [8], [-4], [-18], [-8], [4], [-2], [18], [12], [4], [-22], [0], [-16], [12], [-8], [-4], [6], [2], [6], [-24], [-16], [-22], [20], [0], [30], [6], [-16], [12], [18], [6], [-28], [2], [24], [8], [-12], [-28], [-16], [-22], [-2], [0], [-8], [10], [20], [-20], [6], [30], [36], [-14], [14], [-12], [2], [16], [0], [-24], [6], [6], [-24], [18], [-26], [18], [32], [2], [4], [4], [-34], [-34], [10], [-10], [2], [34], [-12], [-18], [-18], [20], [32], [-38], [-16], [40], [18], [10], [20], [-8], [-4], [-40], [-6], [-18], [2], [-36], [36], [-14], [-16], [12], [38], [-36], [-22], [38], [16], [18], [-36], [-12], [30], [-16], [-52], [-38], [-2], [20], [2], [-12], [-42], [36], [18], [-2], [42], [-10], [24], [-28], [-8], [44], [-42], [-44], [10], [-50], [18], [-22], [48], [-24], [38], [-40], [-14], [-30], [-24], [50], [36], [32], [46], [-32], [12], [14], [36], [-20], [-30], [-24], [40], [-16], [-26], [-12], [58], [14], [6], [-38], [12], [-12], [24], [6], [-10], [4], [14], [-2], [0], [32], [54], [-4], [-54], [18], [8], [10], [44], [-64], [-50], [-18], [16], [28], [-2], [-4], [4], [-32], [-50], [-10], [-28], [-32], [18], [46], [-22], [-40], [-62], [-22], [50], [34], [-20], [34], [40], [30], [54], [2], [-50], [24], [-64], [-46], [-60], [-4], [-60], [0], [2], [-26], [42], [-10], [-60], [-20], [72], [-2], [-44], [-34], [46], [-48], [-56], [-38], [12], [60], [-26], [-12], [-30], [-30], [-2], [-52], [-60], [10], [10], [-4], [24], [-74], [-62], [38], [28], [48], [-12], [-50], [-70], [6], [36], [-46], [82], [-8], [64], [-48], [46], [-38], [30], [66], [28], [-10], [40], [-62], [26], [-70], [-48], [32], [-36], [40], [70], [26], [2], [56], [36], [12], [-42], [32], [-12], [76], [2], [56], [-44], [-16], [30], [52], [48], [42], [14], [-24], [-28], [54], [54], [34], [-2], [-14], [54], [36], [32], [0], [86], [-4], [88], [-86], [-42], [-8], [-22], [20], [-90], [10], [-44], [26], [68], [64], [14], [36], [26], [-10], [-64], [-30], [90], [52], [-40], [62], [40], [60], [-12], [2], [-6], [48], [92], [44], [8], [90], [-42], [-74], [-8], [64], [4], [-64], [14], [44], [42], [74], [12], [-14], [-56], [36], [-34], [50], [12], [-100], [6], [46], [88], [-76], [4], [-96], [-32], [22], [88], [-30], [-40], [2], [32], [52], [78], [48], [-20], [-26], [-54], [-90], [36], [-4], [-14], [-40], [-88], [-32], [-42], [36], [50], [54], [80], [42], [-104], [-66], [-88], [-28], [-46], [-34], [-76], [80], [12], [6], [-20], [56], [50], [-96], [50], [-64], [-82], [-74], [-52], [-102], [12], [24], [84], [-34], [-82], [52], [8]]
